from __future__ import print_function

import argparse
import tasks
import logging
import json

import packages
import config

from helpers import ObjectEncoder
from context import Context

SCRIPT_VERSION = "1.0.6"
INSTALL_ONLY_CAPTURECAST = False

LOG_FORMAT = "%(asctime)s [%(name)s] %(levelname)s: %(message)s"
LOG_DATE_FORMAT = "%Y-%m-%d.%I-%M-%S-%p"

logger = logging.getLogger(__name__)
file_handler = None

def configure_logging (config):
    global file_handler

    # basic log configuration
    logging.basicConfig(
        format=LOG_FORMAT,
        datefmt=LOG_DATE_FORMAT,
        level=config.loglevel.upper() if config.loglevel else "INFO"
    )

    # log file configuration
    if file_handler:
        logging.getLogger().removeHandler(file_handler)

    file_handler = logging.FileHandler(config.logfile)
    file_handler.setFormatter(logging.Formatter(fmt=LOG_FORMAT, datefmt=LOG_DATE_FORMAT))
    logging.getLogger().addHandler(file_handler)

def parse_args (args):
    parser = argparse.ArgumentParser(description="Cattura CaptureCast Pro Installer, version {}".format(SCRIPT_VERSION))

    # Actions
    for action_name, value in config.options["actions"].iteritems():
        help = value["title"]
        if "description" in value:
            help = "{}: {}".format(help, value["description"])
        names = ["--{}".format(action_name)]
        if "abbreviation" in value:
            names = ["-{}".format(value["abbreviation"])] + names
        parser.add_argument(*names, type=str, nargs="*", help=help)

    # Action options
    parser.add_argument("-l", "--loglevel", type=str, help="Log level (default INFO)", nargs="?")
    parser.add_argument("-lf", "--logfile", type=str, help="Log file (default installer-<datetime>.log)", nargs="?")
    parser.add_argument("-d", "--dry", action="store_true", help="Dry run: don't make any changes to the system")
    parser.add_argument("-j", "--json", action="store_true", help="Output result value as json")
    parser.add_argument("-m", "--mode", type=str, help="Mode to run installer in (default terminal)", nargs="?")
    parser.add_argument("-y", "--yes", action="store_true", help='Automatically answer "yes" to any prompts')

    # Repository to track
    parser.add_argument("--repo", "--version", type=str, help="repository name or URL", nargs="?")
    parser.add_argument("--key", type=str, help="license key for repo", nargs="?")

    # Database settings
    parser.add_argument("--database-type", type=str, help="type of database", nargs="?")
    parser.add_argument("--database-user", type=str, help="database username", nargs="?")
    parser.add_argument("--database-name", type=str, help="database name", nargs="?")
    parser.add_argument("--database-password", type=str, help="database password", nargs="?")

    # Feature options
    parser.add_argument("--publishers", type=str, nargs="*", help="Publishers to install (default is standard publishers)")
    parser.add_argument("--features", type=str, nargs="*", help="Features to install (default is standard features)")
    parser.add_argument("--drivers", type=str, nargs="*", help="Drivers to install (default is all drivers)")

    # Installation options
    parser.add_argument("-rf", "--restore-from", type=str, help="Backup file to restore from after install", nargs="?")

    return config.from_parsed_args(parser.parse_args(args))

class Installer (object):
    def __init__ (self, config):
        self.config = config

    def setup_os (self):
        self.remount()
        tasks.setup_user(self.config)
        tasks.setup_os(self.config)

    def register (self):
        self.remount()
        retval = tasks.register(self.config)

        if not INSTALL_ONLY_CAPTURECAST:
            tasks.register_extra(self.config)

        return retval

    def install (self):
        self.remount()
        CATTURA_PLATFORM_PACKAGE = "cattura-capturecast-platform"

        tasks.remove_legacy_platform(self.config) # "legacy" = anything < 3016
        self.register()
        tasks.debconf(self.config)

        if not INSTALL_ONLY_CAPTURECAST:
            self.setup_os()

        platformInstalled = len([package for package in packages.getSystemPackages() if package.package == CATTURA_PLATFORM_PACKAGE and "installed" in package.status]) > 0

        if not platformInstalled:
            tasks.install_database(self.config)

        tasks.install_platform(self.config)
        self.install_drivers()

        if not platformInstalled:
            tasks.install_features(self.config)
        else:
            tasks.update_platform(self.config)
            if self.config.features or self.config.publishers:
                tasks.install_features(self.config)

        if not INSTALL_ONLY_CAPTURECAST:
            self.install_extras()

        tasks.restore_backup(self.config)

        self.check()

    def install_command_center (self):
        self.remount()
        tasks.register_extra(self.config)
        return tasks.install_cranium(self.config)

    def get_registration_info (self):
        return tasks.get_registration_info(self.config)

    def check (self):
        return tasks.afterinstall_checks(self.config)

    def check_updates (self):
        self.remount()
        return tasks.check_updates(self.config)

    def uninstall (self, *args):
        self.remount()
        return tasks.remove_platform(self.config)

    def downgrade (self, *args):
        self.remount()
        tasks.remove_platform(self.config)
        self.install()

    def reinstall (self, *args):
        self.remount()
        tasks.remove_platform(self.config)
        self.install()

    def unregister (self, *args):
        self.remount()
        return tasks.unregister(self.config)

    def wipe_data (self, *args):
        return tasks.remove_data(self.config)

    def nuke (self, *args):
        self.remount()

        try:
            self.uninstall()
        except Exception:
            logger.warning("Failed to uninstall platform, perhaps because it was not installed")

        try:
            self.unregister()
        except Exception:
            logger.warning("Failed to unregister recorder, perhaps because it was not registered")

        try:
            self.wipe_data()
        except Exception:
            logger.warning("Failed to wipe data, perhaps because there was no data to wipe")

        return tasks.nuke(self.config)

    def install_extras (self):
        self.remount()
        return tasks.install_extras(self.config)

    def install_drivers (self):
        self.remount()
        return tasks.install_drivers(self.config)

    def create_user (self):
        self.remount()
        tasks.register_extra(self.config)
        return tasks.setup_user(self.config)

    def remove_user (self):
        self.remount()
        tasks.remove_user(self.config)

    def refresh_user (self):
        self.remount()

        try:
            tasks.remove_user(self.config)
        except Exception:
            logger.warning("Failed to remove user, perhaps because user did not exist")

        context = Context(self.config, logger)
        context.apt_update()

        tasks.setup_user(self.config)

    def remount (self):
        tasks.remount(self.config)

    def __call__ (self):
        retvals = {}
        for action in self.config.actions:
            method_name = action["action"].replace("-", "_")
            logger.info("Running action: {}".format(action["action"]))
            retvals[action["action"]] = retval(vars(Installer)[method_name](self, *action["args"]))
        return retvals

def retval (value):
    return True if value is None else value

def main (args=None, user_config=None):
    if not args:
        args = []

    if not user_config:
        user_config = parse_args(args)

    configure_logging(user_config)

    try:
        logger = logging.getLogger(__name__)
        logger.info("Cattura CaptureCast Pro Installer, version {}".format(SCRIPT_VERSION))
        logger.info("Config: {}".format(user_config))
        if user_config.dry:
            logger.info("Performing a dry run")

        retvals = Installer(user_config)()

        if user_config.json:
            print(json.dumps(retvals, cls=ObjectEncoder))
    except Exception as e:
        if user_config.json:
            print(json.dumps({
                "err": {
                    "type": type(e).__name__,
                    "message": e.message
                }
            }))
        else:
            raise
