import os
import urlparse
from subprocess import CalledProcessError

def get_database (context, database_config):
    database_config = database_config or context.config.database
    if database_config["type"] == "postgresql":
        return Postgresql(context, database_config)

def get_connection_info (cattura_config):
    if cattura_config and "database-connection" in cattura_config:
        connection = urlparse.urlparse(cattura_config["database-connection"])
        name = connection.path.split("/")[-1]
        params = urlparse.parse_qs(connection.query)
        return {
            "type": "postgresql" if connection.path.startswith("postgresql://") else "sqlite",
            "name": name,
            "user": params["user"][0],
            "password": params["password"][0]
        }

class Postgresql (object):
    def __init__ (self, context, config):
        self.context = context
        self.config = config
    
    def install (self):
        self.context.install("postgresql")
        self.__auth_config_fix()
        self.__setup_database()
    
    def drop (self):
        self.context.call(["dropdb", self.config["name"]], env={"PGPASSWORD": self.config["password"], "PGUSER": self.config["user"]})
    
    def __auth_config_fix (self):
        etc_postgres = os.path.join("/","etc","postgresql")
        if not os.path.isdir(etc_postgres):
            return False
        
        was_fixed = False
        for version in os.listdir(etc_postgres):
            hba_conf_path = os.path.join(etc_postgres,version,"main","pg_hba.conf")
            hba_conf_contents = []
            
            # fix hba configuration
            for line in self.context.read_privileged(hba_conf_path).split("\n"):
                if line.startswith("local "):
                    lineparts = [element for element in line.split(" ") if not element == '']
                    if lineparts[1] == 'all' and lineparts[2] == 'all':
                        line = "local   all             all                                     md5"
                        was_fixed = True
                hba_conf_contents.append(line.strip())
            
            if was_fixed:
                self.context.write_privileged("\n".join(hba_conf_contents), hba_conf_path)
                self.context.logger.info("Fixed {}".format(hba_conf_path))
        
        if was_fixed:
            self.context.call_privileged(["service", "postgresql", "restart"])
        
        return was_fixed
    
    def __setup_database (self):
        try:
            self.context.logger.info("Creating postgresql user {}".format(self.config["user"]))
            self.context.call_privileged(["sudo", "-u", "postgres", "psql", "-c", "CREATE USER {} WITH SUPERUSER PASSWORD '{}'".format(self.config["user"], self.config["password"])])
        except CalledProcessError as e:
            self.context.logger.warn("Unable to create postgresql user {}: {} (maybe user already exists?)".format(self.config["user"], e))
            #self.context.call(["psql", "-c", "SELECT 1"], env={"PGPASSWORD": self.config["password"], "PGUSER": self.config["user"]})
        
        #logger.info("Creating postgresql database {}".format(db_config["name"]))
        #context.call_privileged(["sudo", "-u", "postgres", "createdb", "-O", db_config["user"], db_config["name"]])
    
    def get_connection_string (self):
        return "jdbc:postgresql://127.0.0.1/{name}?user={user}&password={password}".format(**self.config)