import os
from StringIO import StringIO
from urlparse import urlparse
import urllib2
import gzip
import base64

DPKG_STATUS = os.path.join("/", "var", "lib", "dpkg", "status")
PACKAGES = "Packages"
PACKAGES_GZ = "Packages.gz"

class Repo (object):
    def __init__ (self, type, url, dist):
        self.type = type
        self.url = url
        self.dist = dist

    def packages (self):
        packageData = ""

        try:
            packageData = urlopen(self.url + self.dist + PACKAGES).read()
        except urllib2.URLError:
            packageData = gzip.GzipFile(fileobj=StringIO(urlopen(self.url + self.dist + PACKAGES_GZ).read())).read()

        return parsePackageList(packageData)

def getReposFromFile (file):
    repos = []
    for line in open(file, "r").read().strip().split("\n"):
        if (not line) or line.startswith("#"):
            continue

        parts = line.split(" ")
        repos.append(Repo(
            type = parts[0],
            url = parts[1],
            dist = parts[2]
        ))
    return repos

class Package (object):
    def __init__ (self, data):
        self.package = data["Package"]
        self.description = data.get("Description", "")
        self.maintainer = data.get("Maintainer", "")
        self.section = data.get("Section", "")
        self.md5sum = data.get("MD5sum", "")
        self.installedSize = data.get("Installed-Size", 0)
        self.depends = [depend.strip() for depend in data.get("Depends", "").split(",")]
        self.version = data.get("Version", "")
        self.architecture = data.get("Architecture", "")
        self.size = data.get("Size", 0)
        self.status = data.get("Status", "")
        self.priority = data.get("Priority", "")

    def __repr__ (self):
        return 'Package({"Package": %r, "Description": %r, "Maintainer": %r, "Section": %r, "MD5sum": %r, "Installed-Size": %s, "Version": %r, "Architecture": %r, "Size": %r, "Status": %r, "Priority": %r})' % (self.package, self.description, self.maintainer, self.section, self.md5sum, self.installedSize, self.version, self.architecture, self.size, self.status, self.priority)

def parsePackageList (packageData):
    packages = []
    currentPackageData = {}
    key = None
    value = None
    for line in packageData.strip().split("\n"):
        if line.startswith(" "):
            value = value + " " + line
        else:
            if key is not None:
                currentPackageData[key] = value
            if ":" in line:
                parts = line.split(":")
                key = parts[0]
                if len(parts) > 1:
                    value = parts[1].strip()
                else:
                    value = ""

        if len(line.strip()) == 0:
            packages.append(Package(currentPackageData))
            currentPackageData = {}

    # 8 December 2017: append any leftover package data
    if currentPackageData and currentPackageData["Package"]:
        packages.append(Package(currentPackageData))

    return packages

def getSystemPackages ():
    with open(DPKG_STATUS) as dpkgStatus:
        return parsePackageList(dpkgStatus.read())

def urlopen (url):
    ((username, password), url) = extract_auth(url)
    request = urllib2.Request(url)

    if not (username is None and password is None):
        base64string = base64.b64encode('%s:%s' % (username, password))
        request.add_header("Authorization", "Basic %s" % base64string)

    return urllib2.urlopen(request)

def extract_auth (url):
    parsed = urlparse(url)
    url = "{}://{}{}{}".format(parsed.scheme, parsed.hostname, ":{}".format(parsed.port) if parsed.port else "", parsed.path)
    return ((parsed.username, parsed.password), url)

class Changes (object):
    def __init__ (self, changes):
        self.changes = changes or []

    def __repr__ (self):
        return "Changes({})".format(self.changes)

    def get_changes_since (self, version):
        changes_since = []
        for changes in self.changes:
            if changes.version == version:
                break
            changes_since.append(changes)
        return changes_since

class Changeset (object):
    def __init__ (self, package, version, changes=None):
        self.package = package
        self.version = version
        self.changes = changes or []

    def __repr__ (self):
        return "Changeset({}, {})".format(self.package, self.version)

def parseChangelog (changelog):
    changes = []
    changeset = None

    for line in changelog.split("\n"):
        line = line.strip()
        if not line:
            continue

        if changeset is None:
            (package, version, etc) = line.split(" ", 2)
            version = version[1:-1]
            changeset = Changeset(package, version)
        elif line.startswith("*"):
            changeset.changes.append(line[1:].strip())
        elif line.startswith("--"):
            changes.append(changeset)
            changeset = None
        elif changeset.changes:
            changeset.changes[-1] = changeset.changes[-1] + " " + line

    return Changes(changes)
