from subprocess import check_output, CalledProcessError
import logging
import os

from ..context import Context
from ..capturecast import CaptureCast

logger = logging.getLogger(__name__)

DRIVER_NAMES = ["ProCapture"]
VIDEO_DEVICES_DIR = "/dev/cattura"

def __call__ (config):
    context = Context(config, logger)
    capturecast = CaptureCast(context)
    result = {}
    
    result["drivers"] = check_drivers()
    result["video_devices"] = check_video_devices()
    result["plugins"] = check_plugins(capturecast)
    return result

def check_drivers ():
    module_names = list_modules()
    modules = {}

    for name in DRIVER_NAMES:
        try:
            modinfo = get_modinfo(name)
            modules[name] = modinfo
            logger.info("Module %s (version %s) found", name, modinfo['version'])
            if not name in module_names:
                logger.warn(" -- Module %s not loaded!", name)
        except CalledProcessError as e:
            logger.warn("Could not find module: %s; %s", name, e)
    return modules

def check_video_devices ():
    devices = []
    try:
        devices = os.listdir(VIDEO_DEVICES_DIR)
        logger.info("Found %s devices under %s: %s", len(devices), VIDEO_DEVICES_DIR, ", ".join(devices))
        if not devices:
            logger.warn(" -- No devices found!")
    except OSError as e:
        logger.warn("Could not list directory %s; %s", VIDEO_DEVICES_DIR, e)
    return devices

def check_plugins (capturecast):
    all_plugins = {}
    try:
        for subdirectory in os.listdir(capturecast.get_plugins_directory()):
            full_path = os.path.join(capturecast.get_plugins_directory(), subdirectory)
            plugins = os.listdir(full_path)
            all_plugins[subdirectory] = plugins
            logger.info("Found %s plugins under %s: %s", len(plugins), full_path, ", ".join(plugins))
            if not plugins:
                logger.warn(" -- No %s plugins found!", subdirectory)
    except OSError as e:
        logger.warn("Could not list directory %s; %s", capturecast.get_plugins_directory(), e)
    return all_plugins

def get_modinfo (name):
    modinfo = {}
    for line in check_output(["/sbin/modinfo", name]).split("\n"):
        if ":" in line:
            (key, value) = line.split(":", 1)
            modinfo[key] = value.strip()
    
    return modinfo

def list_modules ():
    module_names = []
    for line in check_output(["lsmod"]).split("\n"):
        module_names.append(line.split(" ")[0])
    return module_names[1:]