import logging

from subprocess import check_output, CalledProcessError, PIPE
from urllib2 import HTTPError

from ..context import Context
from ..packages import getReposFromFile, urlopen, parseChangelog

logger = logging.getLogger(__name__)

CATTURA_APT_FILE = "/etc/apt/sources.list.d/cattura.list"

def __call__ (config):
    context = Context(config, logger)
    
    logger.info("Checking for updates to Cattura Platform")
    
    context.apt_update()
    
    for repo in getReposFromFile(CATTURA_APT_FILE):
        logger.info("Fetching package list from [{}]".format(repo.url))
        cattura_packages = repo.packages()
        
        logger.info("{} total Cattura packages found.".format(len(cattura_packages)))
        
        logger.info("Finding packages that need to be updated")
        all_updates = get_all_updates()
        cattura_updates = filter_updates_for_packages(all_updates, cattura_packages)
        
        logger.info("Collecting changelogs...")
        cattura_updates = collect_changelogs(cattura_updates, repo)
        
        logger.info("{} updates to Cattura packages found.".format(len(cattura_updates)))
        log_updates(cattura_updates)
        
        return cattura_updates

def get_all_updates ():
    """Return all packages that need an update."""
    updates = {}
    for line in check_output(["apt-get", "upgrade", "-s"]).split("\n"):
        if not line.startswith("Inst "):
            continue

        (inst, package, current_version, new_version, etc) = line.split(" ", 4)
        updates[package] = {
            "package": package,
            "current_version": current_version[1:-1],
            "new_version": new_version[1:]
        }
    return updates

def filter_updates_for_packages (updates, packages):
    filtered_updates = []
    for package in packages:
        if package.package in updates:
            update = {}
            update.update(updates[package.package])
            update["package"] = package
            filtered_updates.append(update)
    return filtered_updates

def collect_changelogs (updates, repo):
    for update in updates:
        changelog = get_changelog_with_apt(update["package"]) or get_changelog_from_repo(update["package"], repo)
        if changelog:
            update["changes"] = parseChangelog(changelog).get_changes_since(update["current_version"])
    return updates

def log_updates (updates):
    for update in updates:
        logger.info(" - {package.package} ({current_version} -> {new_version})".format(**update))
        if "changes" in update:
            for changeset in update["changes"]:
                logger.info("  - {}".format(changeset.version))
                for change in changeset.changes:
                    logger.info("   * {}".format(change))

def get_changelog_with_apt (package):
    try:
        return check_output(["apt-get", "-qq", "changelog", package.package], stderr=PIPE)
    except CalledProcessError:
        return None

def get_changelog_from_repo (package, repo):
    try:
        return urlopen("{}/{}_{}.changelog".format(repo.url, package.package, package.version)).read()
    except HTTPError:
        return None