import os
import logging
from urlparse import urlparse
from bundle import open_entry

from ..config import options
from ..context import Context

from ..registration import CATTURA_APT_FILE, get_unit_id, discover_cranium

from turbo_activate import TurboActivate, PDetsFromByteArray, USER, OK, ActivateOptions

logger = logging.getLogger(__name__)

DEFAULT_REPO_URL = "http://repositories.catturavideo.com/"
DEFAULT_REPO = "production".format(DEFAULT_REPO_URL)

CAPTURECAST_PRO_GUID = "50d602e6586edb97afc8e4.97968303" # for TurboActivate integration

def __call__ (config):
    context = Context(config, logger)
    
    # set up cattura repo
    # we only want to if
    #  a) this is first time set up (repo is not set up)
    #  b) user has specified a specific repo (which may indicate switching to another repo)
    # default repo is the stable/production one
    if context.config.repo or not os.path.exists(CATTURA_APT_FILE):
        repo_url = get_full_repo_url(context.config.repo if context.config.repo else DEFAULT_REPO)
        unit_info = register_recorder(context, repo_url)

        if unit_info:
            repo_url = add_unit_authentication(repo_url, unit_info)
        
        logger.info("Adding Cattura repository to apt")
        context.write_privileged("deb {} /".format(repo_url), CATTURA_APT_FILE)
    else:
        logger.info("Cattura repository already set up, nothing to do here")
    
    # Add Oracle Java repository
    logger.info("Adding webupd8 java repository to apt")
    context.call_privileged(["add-apt-repository", "-y", "ppa:webupd8team/java"])
    
    # update apt-get
    context.apt_update()

def get_full_repo_url (repo_name):
    if repo_name in options["repos"]:
        repo_name = options["repos"][repo_name]
    
    if not (repo_name.startswith("http://") or repo_name.startswith("https://")):
        repo_name = "{}{}".format(DEFAULT_REPO_URL, repo_name)
    
    return repo_name

def register_recorder (context, repo_url):
    cranium = discover_cranium(repo_url)
    if not cranium:
        logger.info("Skipping registration since there doesn't seem to be a valid registration endpoint...")
        return None
    
    unit_id = get_unit_id()
    logger.info('Registering unit with id "{}"'.format(unit_id))
    
    cranium.token = context.config.key
    
    if not cranium.token:
        cranium.token = context.input(" >> Enter registration key: ")
    
    cranium_user = None
    
    try:
        cranium_user = cranium.get_user("me")
    except Exception:
        raise Exception("Invalid registration key provided")
    
    if context.config.dry:
        return {
            "unit": {"id": unit_id},
            "token": "jeff"
        }
    
    prospective_unit_id = get_unit_id()
    unit = None
    try:
        unit = cranium.add_unit({
            "url": "http://{}".format(cranium.client_info()["ip"])
        }, unit_id=prospective_unit_id)
    except Exception as e:
        logger.info('Failed to create unit "{}" ({}); assuming unit already exists'.format(prospective_unit_id, e))
        unit = cranium.get_unit(prospective_unit_id)
    
    # If we're a LimeLM key, use the TurboActivate flow
    if "pkey" in cranium_user:
        return register_with_turbo_activate(cranium, unit)
    else:
        token = cranium.add_unit_token(unit["id"])
        return {
            "unit": unit,
            "token": token["inboundAccessToken"]
        }

def register_with_turbo_activate (cranium, unit):
    turbo_activate = initialize_turbo_activate()
    
    # Generate activation request for TurboActivate, including our unit id.
    turbo_activate.CheckAndSavePKey(cranium.token, USER)
    activation_request = turbo_activate.ActivationRequestToFile("TurboActivateRequest.xml", ActivateOptions(unit["id"]))
    
    with open("TurboActivateRequest.xml") as activation_request:
        cranium.activate_unit(unit["id"], activation_request.read())
    
    # TurboActivate units use the LimeLM product key to authenticate
    return {
        "unit": unit,
        "token": cranium.token
    }

def add_unit_authentication (repo_url, unit_info):
    parsed = urlparse(repo_url)
    return "{}://{}:{}@{}{}".format(
        parsed.scheme,
        unit_info["unit"]["id"],
        unit_info["token"],
        parsed.netloc,
        parsed.path
    )

def initialize_turbo_activate ():
    with open_entry("TurboActivate.dat") as pdets_file:
        pdets_data = pdets_file.read()
        if PDetsFromByteArray(pdets_data, len(pdets_data)) != OK:
            raise Exception("PDetsFromByteArray failed")
    return TurboActivate(CAPTURECAST_PRO_GUID)